import argparse
import json
import sys

from pathlib import Path

from kaas.parser import Parser

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('file', help='File to convert')
    return parser.parse_args()

def main():
    args = parse_args()
    fp = Path(args.file)
    if not fp.exists():
        print(f"File {fp} does not exist!", file=sys.stderr)
        sys.exit(1)
    if not fp.is_file():
        print(f"Path {fp} is not a file!", file=sys.stderr)
        sys.exit(1)
    text = fp.open().read()
    p = Parser().parse_text(text)
    print(json.dumps(p, indent=4))



import logging
from pyleri import (
    Grammar,
    Token,
    Regex,
    Sequence,
    Ref,
    Choice,
    Repeat,
    )


logger = logging.getLogger()

class PSGrammar(Grammar):
    LIST = Ref()
    OBJ = Ref()
    STRING = Regex(r"'(?:.)*?'")
    INT = Regex("\d+")
    BOOL = Choice(Token("$False"), Token("$True"))
    ni_item = Choice(OBJ, LIST, STRING, BOOL, INT)
    variable = Sequence(Regex('\w+'), Token('='), ni_item)
    LIST = Sequence('@(', Repeat(ni_item), ')')
    OBJ = Sequence('@{', Repeat(variable), '}')
    START = Choice(OBJ, LIST)

class Parser:
    def __init__(self):
        self.parser = PSGrammar()

    def parse_node(self, node):
        logger.debug(node.element)
        if isinstance(node.element, Repeat):
            logger.debug('repeat x%s', len(node.children))
            return [self.parse_node(c) for c in node.children]
        elif isinstance(node.element, Choice) and getattr(node.element, 'name') != 'BOOL':
            assert len(node.children) == 1
            return self.parse_node(node.children[0])

        if not hasattr(node.element, 'name'):
            logger.debug('%s!!', node.element)
            return
        if node.element.name in 'OBJ':
            elements = self.parse_node(node.children[1])  # 0 and 2 are @{ }
            _dict = {}
            for k, v in elements:
                _dict[k] = v
            return _dict
        elif node.element.name == 'variable':
            name_node, equals, value_node = node.children
            var_value = self.parse_node(value_node)
            var_name = name_node.string
            logger.debug(f'Variable {var_name} == {var_value}')
            return var_name, var_value
        elif node.element.name == 'INT':
            return int(node.string)
        elif node.element.name == 'STRING':
            return node.string.strip("'")
        elif node.element.name == 'BOOL':
            return node.string == '$True'
        elif node.element.name == 'LIST':
            # 0 and 2 are [ ]
            return [self.parse_node(n) for n in node.children[1].children]
        else:
            logger.debug('unk %s', node.element)
        logger.debug('Elem: %s, Name: %s, Val: %s', node.element, node.element.name, node.string)
        assert False


    def parse_text(self, text):
        val = self.parser.parse(text)
        root = val.tree.children[0]
        res = self.parse_node(root)
        return res



#!/usr/bin/env python3

from setuptools import setup

setup(
    name="kaas",
    version="0.0.1",
    packages=["kaas"],
    long_description="Parse PowerShell PSD1 files",
    url="https://github.com/DavidVentura/kaas",
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    install_requires=[],
    entry_points={"console_scripts": ["psd2json = kaas.cli:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
    )
